/*  "hash cash" mint and remailer plug in for purposes of
 *  remailer postage (or any other use you can think of)
 * 
 *  hash cash is payment in burnt CPU cycles by calculating n-bit
 *  partial hash collisions
 *
 *  see:         http://www.dcs.ex.ac.uk/~aba/hashcash/
 *
 *  for updates.
 *
 *  version 0.1: 27 Mar 97 
 *               Adam Back <aba@dcs.ex.ac.uk>
 *               http://www.dcs.ex.ac.uk/~aba/hashcash/
 *
 *  version 0.2: 28 Mar 97
 *               Chris Kuethe <ckuethe@gpu.srv.ualberta.ca>
 *               http://www.ualberta.ca/~ckuethe/
 *               added minimal macintosh support (Think C)
 *
 *  version 0.3: 30 Mar 97 
 *               Adam Back <aba@dcs.ex.ac.uk>
 *               put in fast SHA1, and other mods giving a 4x speedup
 *               plus a few bug fixes
 *
 *  version 0.4  31 Mar 97
 *               Adam Back <aba@dcs.ex.ac.uk> &
 *               More portable timer and types macros
 *
 *  version 0.5  07 Apr 97
 *               Adam Back <aba@dcs.ex.ac.uk> &
 *               made mods as suggested by
 *                 Andy Dustman <andy@CCMSD.chem.uga.edu>
 *               to avoid collision collisions on larger collision lengths
 *               previous method was bugged
 *
 *  version 0.6  07 May 97
 *               Andy Dustman <andy@CCMSD.chem.uga.edu>
 *               bug fix + SPARSE compile option for library use
 *
 *  version 0.7u: 01 June 1997
 *               Command-Line args etc tidied.  ant@notatla.demon.co.uk
 *               Made less verbose, trimmed some code.
 *               Sender now chooses validity period, (start & duration)
 *               and remailer checks both.
 *
 */

#include "hashcash.h"

void random_start (word32 *, word32 *, word32 *, word32 *);
void usage (void);
word32 find_collision (char day[5], char *, int, char *, word32, char *,int);
double usecs_for_n_hashes (word32,int);
/*** void add_to_database (char*, char *, int);  ***/
void hashcash_purge_database (char *);

/* a 120Mhz 486 does 28000 per second

   do a special case for tick = 1 second, say 5 seconds
   so 131071 tries
 */


word32 hashes_per_sec (int);


int bits = 20;
char today[6];
word32 today_num;

int
main (int argc, char *argv[])
{
  int tries;
  int i;
  word32 i0, i1, i2;
  int done0 = 0, done1 = 0, done2 = 0;
  int collision_bits;
  int arguments = 0;
  int mint_flag = 0;
  int spend_flag = 0;
  int bits_flag = 0;
  int maxdays = 0;
  int validity_flag = 0;
  int database_flag = 0;
  int name_flag = 0;
  int validity_period = 28;
  int valid, in_db, gig, expired, seconds;
  char collision[100];
  char name[100];
  char base[100];
  char check[100];
  double tries_expected, tries_taken, time_estimated;
  double taken, count;
  char *db_filename = "hashcash.db";

#ifdef THINK_C
  argc = ccommand (&argv);
#endif

/*  Acceptable forms:

 *   hashcash can run in 'time', 'mint' or 'spend' mode.
 *
 * hashcash -t
 *
 * hashcash -m bits check [duration] [now]
 *
 * hashcash -s bits check collision maxdays -d[name]
 *
 *    -dname tells us the database name,
 *     if -d is used alone it means use the default name
 *
 */
  if (argc < 2)
    usage ();
  if (argv[1][0] != '-')
    usage ();

  switch (argv[1][1])
    {
    case 'm':
      mint_flag = 1;
      break;
    case 's':
      spend_flag = 1;
      break;
    case 't':
      if (argc > 2)
	usage ();
      printf ("speed: %lu hashes per sec\n", hashes_per_sec (validity_period));
      exit (0);
      break;
    default:
      fprintf (stderr, "Unxpected option :%c:\n");
      usage ();
      break;
    }

if (!strncmp (argv[argc - 1], "-d", 2)) {
      if (!spend_flag) usage ();
      database_flag = 1;
      if (strlen (argv[argc - 1]) > 2)
	db_filename = argv[argc - 1] + 2;
      else db_filename =strdup("hashcash.default.db");
  
      fprintf(stderr, "database name is %s\n", db_filename);
         
}

  today_num = (time (0) / 3600) / 24;


for (i = 0; i < (argc - database_flag); i++) {
/***  Command Line Args ***/
      switch (i)
	{
	case 0:
	  strncpy (name, argv[i], 100);
	  name_flag = 1;
	  break;
	case 1:
	  /* option, ignore */
	  break;
	case 2:
	  bits = atoi (argv[i]);
	  if ((bits < 1) || (bits > 160))
	    usage ();
	  break;
	case 3:
	  strncpy (base, argv[i], 100);
	  break;
	case 4:
/**  This is the duration (-m) **/
/**  This is the collision string (-s) **/
	  if (mint_flag)
	    {
	      validity_period = atoi (argv[i]);
	      validity_flag = 1;
	      if (validity_period < 0)
		usage ();
	    }
	  if (spend_flag)
              strncpy(collision,argv[i],sizeof(collision));
	  break;
	case 5:
/**  This is the start date of the coin (-m) **/
/**  This is the max acceptable duration (-s) **/
	  if (mint_flag)
	    {
	      today_num = atoi (argv[i]);
	      if (today_num < 0)
		usage ();
	    }
	  if (spend_flag)
	    {
	      maxdays = atoi (argv[i]);
	      validity_flag = 1;
	      if (maxdays < 0)
		usage ();
	    }
	  break;
	default:
	  /* should never get here */
	  usage ();
	  break;
	}
    }

  sprintf (today, "%05d", today_num);
  fflush (stdout);

  if (spend_flag) {
      i=hashcash_spend(today_num,
                       today,
                       base,
                       collision,
                       validity_period,
                       maxdays,
                       bits,
                       database_flag,
                       db_filename);
      fprintf(stderr,"spend status is %d\n",i);
      exit(i);
  }

  if (mint_flag) {
      hashcash_mint(base,validity_period,bits,today,collision);
      exit(0);
  }

  fprintf(stderr,"%s: should never get here\n",name);
  abort();
  }
