#include <stdio.h>
#include "sha1.h"

#define BUFFER_SIZE 1024

int SHA1_file( SHA1_ctx* ctx, char* filename )
{
   FILE* file;
   byte buffer[ BUFFER_SIZE ];
   int bytes_read;

   if ( strcmp( filename, "-" ) == 0 )
   {
      file = stdin;
   }
   else
   {
      file = fopen( filename, "rb" );
      if ( file == 0 )
      {
	 return -1;
      }
   }
   SHA1_init( ctx );
   while ( !feof( file ) )
   {
      bytes_read = fread( buffer, 1, BUFFER_SIZE, file );
      if ( bytes_read < BUFFER_SIZE && ferror( file ) )
      {
	 return -1;
      }
      SHA1_update( ctx, buffer, bytes_read );
   }
   SHA1_final( ctx );
   fclose( file );
   return 0;
}

void SHA1_print_digest( SHA1_ctx* ctx, FILE* file )
{
   int i;

   for ( i = 0; i < 5; i++ )
   {
      fprintf( file, "%08x", ctx->H[ i ] );
   }
}

int main( int argc, char* argv[] )
{
   int i;
   SHA1_ctx ctx;
   byte digest[ 20 ];
   int status;

   if ( argc == 1 )
   {
      SHA1_file( &ctx, "-" );
      SHA1_print_digest( &ctx, stdout );
      printf( "\n" );
   }
   else
   {
      for ( i = 1; i < argc; i++ )
      {
	 status = SHA1_file( &ctx, argv[ i ] );
	 if ( status < 0 )
	 {
	    perror( argv[ i ] );
	    return 1;
	 }
	 else
	 {
	    printf( "%s: ", argv[ i ] );
	    SHA1_print_digest( &ctx, stdout );
	    printf( "\n" );
	 }
      }
   }
   return 0;
}
